Attribute VB_Name = "modSupport"
Option Explicit

'
' modSupport.BAS
'
' routines to support various functions of
' the Address-O-Matic program/addin
'
' note that we use the Win32.TLB Windows
' API Type Library here
'
'

' global constants (all of these are used only in the addin stuff)
'
Public Const WINDOW_CAPTION = "Address-O-Matic"
Public Const APP_NAME = "mike's Address-O-Matic add-in"

' api declares (not in win32.tlb)
Private Declare Function RegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Private Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long

' common controls support
Private m_bInIDE As Boolean
Public Declare Sub InitCommonControls Lib "comctl32.dll" ()



Public Sub Main()
' load our global form instance if we're run
' as a regular application
'
' if we're run as an add-in, "-Embedding" will
' be passed on the command line

    If Command$ = "" Then frmAddressOMatic.Show
    InitCommonControls
    
End Sub


'
' xp common controls support
'
Public Sub SafeShutdown()
' tells windows not to display the GPF dialog
' beyond this point -- we need this under some
' circumstances, so you might as well do it all
' the time
'
   If Not InIDE() Then
      SetErrorMode SEM_NOGPFAULTERRORBOX
   End If
End Sub

Private Property Get InIDE() As Boolean
   Debug.Assert (IsInIDE())
   InIDE = m_bInIDE
End Property

Private Function IsInIDE() As Boolean
   m_bInIDE = True
   IsInIDE = m_bInIDE
End Function




'
' shell functions
'
Public Sub SetAppPathKey(ByVal AppEXEName As String, ByVal AppPath As String)
' create the apppaths key (so we can be run from the command line)
    
    Dim cName As String
    Dim hKey As Long
    
    On Error Resume Next ' in case of reg permission errors
    
    ' fixup filename
    '
    If Right$(AppPath, 1) <> "\" Then AppPath = AppPath & "\"
    cName = AppPath & AppEXEName
    If LCase$(Right$(cName, 4)) <> ".exe" Then
        cName = cName & ".exe"
    Else
        cName = cName & ""
    End If
    
    ' denormalize path
    '
    AppPath = Left$(AppPath, Len(AppPath) - 1)
    
    
    ' add the registry entry
    '
    Call RegCreateKey(HKEY_LOCAL_MACHINE, "Software\Microsoft\Windows\CurrentVersion\App Paths\" & Dir$(cName), hKey)
'    Call RegSetValue(hKey, "", REG_SZ, cName, Len(cName)) ' set default to full path and file name
    Call RegSetValueExStr(hKey, "", ByVal 0&, REG_SZ, cName, Len(cName))
    Call RegSetValueExStr(hKey, "Path", ByVal 0&, REG_SZ, AppPath, Len(AppPath))
    Call RegCloseKey(hKey)

    On Error GoTo 0

End Sub



'
' safe type conversion functions
'
Public Function SafeBoolean(ByVal Value As String, ByVal DefaultValue As Boolean) As Boolean

    On Error GoTo errHand
    
    SafeBoolean = CBool(Value)
    Exit Function

errHand:
    SafeBoolean = DefaultValue
    Exit Function
End Function

Public Function SafeLong(ByVal Value, Optional ByVal Default As Long = -1) As Long
    
    On Error GoTo errHand
    
    SafeLong = CLng(Value)
    Exit Function
    
errHand:
    SafeLong = Default
    Exit Function
End Function

'
' graphics routines
'
Public Sub PaintIconOnBitmap(ByVal SourceIcon As StdPicture, ByRef BackgroundBitmap As StdPicture)
' returns a stdpicure with the icon drawn on the background
'

    Dim hdc As Long
    Dim hBMPOld As Long
    
    hdc = CreateCompatibleDC(ByVal 0&)
    hBMPOld = SelectObject(hdc, BackgroundBitmap.Handle)
    Call DrawIconEx(hdc, 0, 0, SourceIcon.Handle, 16, 16, 0, 0, DI_NORMAL)
    
    Call SelectObject(hdc, hBMPOld)
    Call DeleteDC(hdc)
    
End Sub

'
' random number routines
'
Public Function RandomLongInRange(Optional ByVal Min As Long = 0, Optional ByVal Max As Long = 100) As Long
' also based on the hardcore vb source code, this random number generator
' returns a value between Min and Max.  review those original notes to
' find out more about this algorithm

    Const MAX_RANGE As Long = 2147483645
    Const MAX_INTEGER As Long = 2147483647
    Const A As Long = 48271
    Const Q As Long = (MAX_INTEGER / A)
    Const R As Long = (MAX_INTEGER Mod A)
    
    Static nLastValue As Long
    
    Dim nLow As Long
    Dim nHigh As Long
    Dim nTemp As Long
    Dim nRandom As Long
    
    
    
    ' validate input parameters
    If Abs(Max - Min) > MAX_RANGE Then Err.Raise 380, "Range cannot be > " & MAX_RANGE
    
    ' see if we need to initialize the last value, since we
    ' use it as a starting point for the next value
    '
    If nLastValue = 0 Then nLastValue = timeGetTime
    
    
    ' do the work
    '
    nHigh = nLastValue / Q
    nLow = nLastValue Mod Q
    
    nTemp = (A * nLow) - (R * nHigh)
    If nTemp >= 0 Then
        nLastValue = nTemp
    Else
        nLastValue = nTemp + MAX_INTEGER
    End If
    
    ' adjust and return
    '
    nRandom = nLastValue - 1
    If Min <> 0 Or Max <> MAX_RANGE Then
        If Min < Max Then
            nRandom = Min + ((nLastValue - 1) Mod (Max - Min + 1))
        Else
            nRandom = Max + ((nLastValue - 1) Mod (Min - Max + 1))
        End If
    End If
    
    RandomLongInRange = nRandom

End Function

