Attribute VB_Name = "modIDEMouseWheel"
Option Explicit

'
' modIDEMouseWheel.BAS
'
' specific routines for the mousewheel addin
'
' note that the modMessageHook's MsgHookProc procedure
' calls our DoScroll() function
'
'



' api declares
Private Const WHEEL_DELTA = 120


' registry-related (settings) constants
Private Const APP_CATEGORY = "Microsoft Visual Basic AddIns"
Private Const APP_NAME = "mike's VB6 IDE MouseWheel AddIn"


' settings members
Private bIntelliPoint As Boolean
Private nLinesPerDelta As Long


'
' settings-related functions and fields
'
Public ScrollInWindowOnly As Boolean
Public CtrlKeyScrollsBottom As Boolean
Public UseCursorPosition As Boolean
Public CtrlKeyScrollsOther As Boolean

Public Sub LoadSettings()
' loads settings from both the environment and
' the registry


    bIntelliPoint = modSupport.IntelliPointAvailable
    nLinesPerDelta = modSupport.MouseWheel_ScrollLines

    ScrollInWindowOnly = SafeBoolean(GetSetting(APP_CATEGORY, APP_NAME, "ScrollInWindowOnly", "False"), False)
    CtrlKeyScrollsBottom = SafeBoolean(GetSetting(APP_CATEGORY, APP_NAME, "CtrlKeyScrollsBottom", "False"), False)
    UseCursorPosition = SafeBoolean(GetSetting(APP_CATEGORY, APP_NAME, "UseCursorPosition", "False"), False)
    CtrlKeyScrollsOther = SafeBoolean(GetSetting(APP_CATEGORY, APP_NAME, "CtrlKeyScrollsOther", "False"), False)

End Sub

Public Sub SaveSettings()
' saves settings to the registry

    SaveSetting APP_CATEGORY, APP_NAME, "ScrollInWindowOnly", ScrollInWindowOnly
    SaveSetting APP_CATEGORY, APP_NAME, "CtrlKeyScrollsBottom", CtrlKeyScrollsBottom
    SaveSetting APP_CATEGORY, APP_NAME, "UseCursorPosition", UseCursorPosition
    SaveSetting APP_CATEGORY, APP_NAME, "CtrlKeyScrollsOther", CtrlKeyScrollsOther

End Sub



'
' main scrolling routine here
'
Public Sub DoScroll(ByRef Message As MSG)
' does the actual scrolling here.  parameter passed ByRef due to VB limitations

    Static nPrevTime As Long
    Static nPrevDelta As Long
    Static hWndPrev As Long
    
    Dim nDelta As Long
    Dim nLines As Long
    Dim nDirection As Long
    
    Dim hWndSB1 As Long
    Dim hWndSB2 As Long
    Dim hWndScrollbar As Long
    
    Dim i As Long
    
    
    
    On Error GoTo errHand
    
    
    ' check for duplicate timestamp, leave as necessary
    '
    If Message.time = nPrevTime Then Exit Sub
    If ShouldScroll(Message) = False Then Exit Sub
    nPrevTime = Message.time
    
    

    ' process wheel delta
    '
    ' include partial delta from last message (if we should),
    ' save partial for next time
    ' - we get partials from the new smooth (ugh!) wheels
    '
    If bIntelliPoint Then
        nDelta = Message.wParam
    Else
        nDelta = HiWord(Message.wParam)
    End If
    
    If Sgn(nDelta) <> Sgn(nPrevDelta) Then nPrevDelta = 0 ' reset previous if we're changing directions
    If hWndPrev <> Message.hWnd Then nPrevDelta = 0       ' reset if we've switched windows
    hWndPrev = Message.hWnd
    
    nDelta = nDelta + nPrevDelta
    nPrevDelta = nDelta Mod WHEEL_DELTA
    nDelta = nDelta \ WHEEL_DELTA
    If nDelta = 0 Then Exit Sub
    
    
    ' get lines to scroll and direction
    '
    nLines = Abs(nDelta * nLinesPerDelta)
    If nDelta < 0 Then
    ' wheel pulled down
        nDirection = SB_LINEDOWN
    Else
    ' wheel pulled up
        nDirection = SB_LINEUP
    End If
    
    
    ' get scrollbar hWnds, and figure out which one to use
    '
    hWndSB1 = FirstChildVerticalScrollbar(Message.hWnd)
    hWndSB2 = NextVerticalScrollbar(hWndSB1)
    If ScrollPrimary(Message, hWndSB1, hWndSB2) Then
        hWndScrollbar = hWndSB1
    Else
        hWndScrollbar = hWndSB2
    End If
    
    
    
    ' scroll the specified number of lines
    '
    For i = 1 To nLines
        SendMessage ByVal Message.hWnd, ByVal WM_VSCROLL, ByVal nDirection, ByVal hWndScrollbar
    Next ' i
    
    ' note that my old VB5 addin code from ~'98 had code in here
    ' to redraw the window.  if you port this code to VB5, you
    ' may need to put something like that back in here
    '
    win.OutputDebugString "VB6 IDE: scrolled " & nLines & " lines."
    Exit Sub
    
    
errHand:
    win.OutputDebugString "DoScroll Error " & Err.Number & ": " & Err.Description
    Resume Next
End Sub

Private Function ShouldScroll(ByRef Message As MSG) As Boolean
' function returns TRUE if the DoScroll() routine should scroll, FALSE if it should not
' logic is based on our settings and the passed parameter
'
' parameter passed ByRef due to VB limitations
'

    Dim rcWindow As RECT
    
    If ScrollInWindowOnly Then
    ' must check to see if the mouse is within the code editor's screen rect
        Call GetWindowRect(Message.hWnd, rcWindow)
        ShouldScroll = CBool(PtInRect(rcWindow, Message.pt.x, Message.pt.y) <> 0)
    Else
        ShouldScroll = True
    End If
    
End Function

Private Function ScrollPrimary(ByRef Message As MSG, ByVal hWndPrimary As Long, ByVal hWndSecondary As Long) As Boolean
' function returns TRUE if the DoScroll() routine should scroll
' the first scrollbar, or FALSE if it should scroll the second,
' based on our settings and the function parameters
'
' hWndPrimary is assumed to be the single/bottom scrollbar, and
' so we expect that it's screen coordinates (if needed) are lower
' than those of hWndSecondary
'
' parameter passed ByRef due to VB limitations

    Dim bCtrlKeyDown As Boolean
    Dim rcPrimary As RECT
    
    Dim bUsePrimary As Boolean
    
    
    If IsVisible(hWndPrimary) And IsVisible(hWndSecondary) Then
    ' both are visible - figure out which one to use based on settings
    
        ' get control key state
        '
        If bIntelliPoint Then
            bCtrlKeyDown = modSupport.CtrlIsHeldDown
        Else
            bCtrlKeyDown = CBool((LoWord(Message.wParam) And MK_CONTROL) <> 0)
        End If
        
        ' determine which one to use
        '
        If UseCursorPosition Then
        ' based on y-coordinate of cursor (remember, secondary is top)
            
            Call GetWindowRect(hWndPrimary, rcPrimary)
            bUsePrimary = CBool(Message.pt.y >= rcPrimary.Top)
                
            ' flip if needed
            If CtrlKeyScrollsOther And bCtrlKeyDown Then bUsePrimary = Not bUsePrimary
            
        Else
        ' based on control key only
            If CtrlKeyScrollsBottom Then
                bUsePrimary = bCtrlKeyDown
            Else
                bUsePrimary = Not bCtrlKeyDown
            End If
        End If
        
    Else
    ' just the one visible - use the primary
        bUsePrimary = True
    End If
    
    ' return the result
    ScrollPrimary = bUsePrimary
    

End Function
