Attribute VB_Name = "modSupport"
Option Explicit

'
' modSupport.BAS
'
' support functions, not related to thread-level hooking,
' for the mousewheel addin
'
'
' UGLY HACK:  this code depends on the vertical
' scrollbar child windows being created in a
' specific order (that is, the first one created
' is the single/split bottom scrollbar, and the second
' one is the split top scrollbar).
'
' UGLY HACK:  since this is a VB-specific addin,
' I don't care about hard-coding the window class
' names into the program, but the reader should
' note that they are there.
'

' API declares (not in win32.tlb)
Private Const SBS_VERT = &H1&
Private Const WHEEL_DELTA = 120&
Private Const WM_MOUSEWHEEL = &H20A&
Private Const SPI_GETWHEELSCROLLLINES = 104&
Private Const SM_MOUSEWHEELPRESENT = 75&
Private Const WM_SETTINGCHANGE = &H1A&

Private Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long



'
' window identification/information routines
'
Public Function IsCodeWindow(ByVal hWnd As Long) As Boolean
    IsCodeWindow = CBool(LCase$(VBGetClassName(hWnd)) = "vbawindow")
End Function

Public Function IsVerticalScrollbar(ByVal hWnd As Long) As Boolean

    Dim nStyle As Long

    If LCase$(VBGetClassName(hWnd)) = "scrollbar" Then
        nStyle = GetWindowLong(hWnd, GWL_STYLE)
        IsVerticalScrollbar = CBool((nStyle And SBS_VERT) <> 0)
    Else
        IsVerticalScrollbar = False
    End If

End Function

Public Function IsVisible(ByVal hWnd As Long) As Boolean
    
    Dim nStyle As Long

    nStyle = GetWindowLong(hWnd, GWL_STYLE)
    IsVisible = CBool((nStyle And WS_VISIBLE) <> 0)

End Function

Public Function FirstChildVerticalScrollbar(ByVal hWndParent As Long) As Long
' returns the hWnd of the first vertical scrollbar window

    Dim hWndChild As Long
    
    hWndChild = GetWindow(hWndParent, GW_CHILD)
    Do
        If IsVerticalScrollbar(hWndChild) Then
            FirstChildVerticalScrollbar = hWndChild
            Exit Function
        Else
            hWndChild = GetWindow(hWndChild, GW_HWNDNEXT)
        End If
    Loop While (hWndChild <> 0)

End Function

Public Function NextVerticalScrollbar(ByVal hWndScrollbar As Long) As Long
' returns the hWnd of the next vertical scrollbar, assuming hWnd

    Dim hWndNext As Long

    Do
        hWndNext = GetWindow(hWndScrollbar, GW_HWNDNEXT)
        If IsVerticalScrollbar(hWndNext) Then
            NextVerticalScrollbar = hWndNext
            Exit Function
        End If
    Loop While (hWndNext <> 0)

End Function



'
' wheel-related functions
'
Public Function IntelliPointHWnd() As Long
' returns the hWnd of the IntelliPoint window
'
' note that this is way old code from 1998 or so, so I
' don't know at what point MS stopped (or if they did)
' supporting this.  I'm assuming that older OSes will
' have old IntelliPoint software, and so this'll work, and
' on newer OSes (98/Nt4+), we'll use the WM_MOUSEWHEEL
' message anyway
'

    IntelliPointHWnd = FindWindow("MouseZ", "Magellan MSWHEEL")

End Function

Public Function IntelliPointAvailable() As Boolean
    IntelliPointAvailable = CBool(IntelliPointHWnd <> 0)
End Function

Public Function MouseWheel_RollMessage() As Long
' returns the message we should be watching for
' in our thread-level hook
    
    If IntelliPointAvailable Then
        MouseWheel_RollMessage = RegisterWindowMessage("MSWHEEL_ROLLMSG")
    Else
        MouseWheel_RollMessage = WM_MOUSEWHEEL
    End If
    
End Function

Public Function MouseWheel_ScrollLines() As Long
' returns the number of lines to scroll on any given
' mousewheel message
'
' intellipoint stuff carries the same caveat as above

    Dim nMsg As Long
    Dim nLines As Long


    If IntelliPointAvailable Then
        nMsg = RegisterWindowMessage("MSH_SCROLL_LINES_MSG")
        nLines = SendMessage(ByVal IntelliPointHWnd, ByVal nMsg, ByVal 0&, ByVal 0&)
    Else
        Call SystemParametersInfo(ByVal SPI_GETWHEELSCROLLLINES, ByVal 0&, nLines, ByVal 0&)
    End If
    
    ' ensure we have a good default
    If nLines = 0 Then nLines = 3
    MouseWheel_ScrollLines = nLines

End Function



'
' generic support routines
'
Public Function CtrlIsHeldDown() As Boolean
    CtrlIsHeldDown = CBool((GetAsyncKeyState(VK_CONTROL) And Not 1) <> 0)
End Function

Public Function VBGetClassName(ByVal hWnd As Long) As String

    Dim cTemp As String
    Dim nLen As Long

    cTemp = String$(255, 0)
    nLen = GetClassName(hWnd, cTemp, Len(cTemp))
    VBGetClassName = Left$(cTemp, nLen)

End Function

Public Function HiWord(ByVal dw As Long) As Integer
    HiWord = (dw And &HFFFF0000) \ 65536
End Function

Public Function LoWord(ByVal dw As Long) As Integer
    If dw And &H8000& Then
        LoWord = dw Or &HFFFF0000
    Else
        LoWord = dw And &HFFFF&
    End If
End Function

Public Function SafeBoolean(ByVal Value As String, ByVal DefaultValue As Boolean) As Boolean

    On Error GoTo errHand
    
    SafeBoolean = CBool(Value)
    Exit Function

errHand:
    SafeBoolean = DefaultValue
    Exit Function
End Function


'
' graphics functions
'
Public Sub PaintIconOnBitmap(ByVal SourceIcon As StdPicture, ByRef BackgroundBitmap As StdPicture)
' returns a stdpicure with the icon drawn on the background
'

    Dim hdc As Long
    Dim hBMPOld As Long
    
    hdc = CreateCompatibleDC(ByVal 0&)
    hBMPOld = SelectObject(hdc, BackgroundBitmap.Handle)
    Call DrawIconEx(hdc, 0, 0, SourceIcon.Handle, 16, 16, 0, 0, DI_NORMAL)
    
    Call SelectObject(hdc, hBMPOld)
    Call DeleteDC(hdc)
    
End Sub

