Attribute VB_Name = "modRegTool"
Option Explicit

'
' modRegTool.BAS
'
' everything you need (execpt the stuff in the references)
' to support registering ActiveX libraries
'
'

' api declares (not in win32.tlb)
Private Declare Function RegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Private Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long

Public Declare Function GetProcAddress Lib "kernel32" (ByVal hModule As Long, ByVal lpProcName As String) As Long
Public Declare Function CreateThread Lib "kernel32" (lpThreadAttributes As Long, ByVal dwStackSize As Long, lpStartAddress As Long, lpParameter As Any, ByVal dwCreationFlags As Long, lpThreadId As Long) As Long
Public Declare Function GetExitCodeThread Lib "kernel32" (ByVal hThread As Long, lpExitCode As Long) As Long
Public Declare Sub ExitThread Lib "kernel32" (ByVal dwExitCode As Long)



'
' main routine
'
Public Sub Main()

    Dim cFiles() As String
    Dim cErrors() As String
    
    Dim bSilent As Boolean
    Dim bUnregister As Boolean
    Dim bInfo As Boolean
    Dim cUnknownFlag As String
    
    Dim i As Long
    
    
    ' get files, set app path support
    cFiles = NormalizedCommandArray
    SetAppPathKey App.EXEName, App.Path
    
    ' show the information dialog, then leave, if nothing was passed
    '
    If LBound(cFiles) > UBound(cFiles) Then
        ShowInformationDialog ""
        Exit Sub
    End If
    
    
    '
    ' if we're here, we're good to go
    '
    '
    
    ' get flags from command line, build error string array
    '
    GetCommandLineFlags cFiles, bSilent, bUnregister, bInfo, cUnknownFlag
    If bInfo Then
        ShowInformationDialog cUnknownFlag
        Exit Sub
    End If
    ReDim cErrors(LBound(cFiles) To UBound(cFiles))
    
    
    ' try to register/unregister each file passed
    '
    For i = LBound(cFiles) To UBound(cFiles)
        If cFiles(i) <> "" Then cErrors(i) = RegisterFile(cFiles(i), bUnregister)
    Next ' i
    
    
    
    ' process the results and leave
    '
    If ProcessResults(bSilent, bUnregister, cFiles, cErrors) Then
    ' had errors
        win.ExitProcess 1
    Else
    ' no errors
        win.ExitProcess 0
    End If
    Exit Sub

End Sub


'
' regtool-specific support routines
'
Private Sub GetCommandLineFlags(ByRef CommandArray() As String, _
                                ByRef Silent As Boolean, ByRef Unregister As Boolean, ByRef Info As Boolean, _
                                ByRef UnknownFlag As String)
' sub reads through the entries in CommandArray() setting the Silent, Unregister, and Info flags as found
' if there is an unknown flag, we'll pass that back, too
' we also pull the flag entries out (by setting the element to "")
'
' note that we expect there is no extraneous space in the array, which is fine if we're passing stuff
' output from NormalizedCommandArray()
'


    Dim i As Long

    ' prep input vars
    '
    Silent = False
    Unregister = False
    Info = False
    UnknownFlag = ""
    
    ' loop through the array, looking for flags
    '
    For i = LBound(CommandArray) To UBound(CommandArray)
        Select Case LCase$(CommandArray(i))
            Case "/s"
                Silent = True
                CommandArray(i) = ""
            Case "/u"
                Unregister = True
                CommandArray(i) = ""
            Case "/?", "/help"
                Info = True
                CommandArray(i) = ""
            Case Else
                If Left$(CommandArray(i), 1) = "/" Then
                    Info = True ' force the messagebox display
                    UnknownFlag = "Unrecognized flag: " & CommandArray(i)
                    CommandArray(i) = ""
                End If
        End Select
    Next ' i

End Sub

Private Sub ShowInformationDialog(ByVal ErrorInfo As String)
' shows a messagebox with our options

    Dim cMessage As String
    Dim nIconIndex As VbMsgBoxStyle


    If ErrorInfo = "" Then
        cMessage = ""
        nIconIndex = vbInformation
    Else
        cMessage = ErrorInfo & vbLf & vbLf
        nIconIndex = vbExclamation
    End If
    
    cMessage = cMessage & "Usage: " & LCase$(App.EXEName) & " [/s] [/u] [/?] libraryname [libraryname...]" & vbLf _
               & vbLf _
               & "   /s    Silent; display no message boxes" & vbLf _
               & "   /u    Unregister server" & vbLf _
               & "   /?    Show instructions (this message)" & vbLf _
               & vbLf _
               & "version " & App.Major & "." & App.Minor & " build " & Format$(App.Revision, "0000")
               
    MsgBox cMessage, nIconIndex, App.EXEName
    
End Sub

Private Function ProcessResults(ByVal Silent As Boolean, ByVal Unregistered As Boolean, ByRef Files() As String, ByRef Errors() As String) As Boolean
' looks through the results to see if everything wenk okay or not.
' function returns True if there were errors, False else
'
' if Silent is False, we'll show a dialog with the results.
' assumes that Files() is a valid (initialized) array, and Errors()
' has the same dimension as Files().
'
' Files() and Errors() passed ByRef due to VB limitations
'


    Dim nSuccessful As Long
    Dim cLastSuccess As String
    Dim nErrors As Long
    Dim cErrors As String
    
    Dim cSuccessMessage As String
    Dim cErrorMessage As String
    
    Dim cRegisterToken1 As String
    Dim cRegisterToken2 As String
    
    Dim nStyle As VbMsgBoxStyle
    
    
    Dim i As Long
    
    
    ' tweak fragments based on registered/unregistered flag
    '
    If Unregistered Then
        cRegisterToken1 = "unregistered"
        cRegisterToken2 = "Unregistration"
    Else
        cRegisterToken1 = "registered"
        cRegisterToken2 = "Registration"
    End If
    
    
    ' count success/errors, build success/error string fragments
    '
    For i = LBound(Files) To UBound(Files)
        If Files(i) <> "" Then
            If Errors(i) = "" Then
                nSuccessful = nSuccessful + 1
                cLastSuccess = Files(i)
            Else
                nErrors = nErrors + 1
                If cErrors <> "" Then
                    cErrors = cErrors & "    " & GetFileName(Files(i)) & " - " & Errors(i) & vbLf
                Else
                    cErrors = GetFileName(Files(i)) & " - " & Errors(i) & vbLf
                End If
            End If
        End If
    Next ' i
    
    ' finish building success/error string message elements
    '
    Select Case nSuccessful
        Case 0
            cSuccessMessage = ""
        Case 1
            cSuccessMessage = "Successfully " & cRegisterToken1 & " " & cLastSuccess & "."
        Case Else
            cSuccessMessage = "Successfully " & cRegisterToken1 & " " & nSuccessful & " items."
    End Select
        
    Select Case nErrors
        Case 0
            cErrorMessage = ""
        Case 1
            cErrorMessage = cRegisterToken2 & " failed for " & cErrors
        Case Else
            cErrorMessage = cRegisterToken2 & " failed for " & nErrors & " items:" & vbLf _
                            & "    " & cErrors
    End Select
    If Right$(cErrorMessage, 1) = vbLf Then cErrorMessage = Left$(cErrorMessage, Len(cErrorMessage) - 1)
    
    
    ' final adjustments to dialog, and handle the "flags only" case
    '
    If (nSuccessful > 0) And (nErrors > 0) Then
    ' need to stick a little bit on the front
        cSuccessMessage = "Done, but with errors." & vbLf & vbLf & cSuccessMessage & vbLf & vbLf
    ElseIf (nSuccessful = 0) And (nErrors = 0) Then
    ' no items passed - this can happen when all we do
    ' is pass flags on the command line
        If Silent = False Then ShowInformationDialog "No files passed."
        ProcessResults = True
        Exit Function
    End If
    
    ' set msgbox style
    If nErrors > 0 Then
        nStyle = vbExclamation Or vbOKOnly
    Else
        nStyle = vbInformation Or vbOKOnly
    End If
    
    
    ' do the message box if we should, and return the result
    '
    If Silent = False Then MsgBox cSuccessMessage & cErrorMessage, nStyle, App.EXEName
    ProcessResults = CBool(nErrors <> 0)

End Function




'
' generic support routines
'
'

'
' registration functions
'
Private Function RegisterFile(ByVal Filename As String, ByVal Unregister As Boolean) As String
' function tries to find an appropriate way to register the passed file

    Dim cExtension As String
    Dim cTLBResult As String
    Dim cDLLResult As String
    
    
    cExtension = LCase$(Right$(Filename, 4))
    
    Select Case cExtension
        Case "tlb", "olb"
        ' these are type library files -- just use the RegisterTypeLibrary function
            RegisterFile = RegisterTypeLibrary(Filename, Unregister)
        Case Else
        ' other files -- try RegisterComponent first, then try RegisterTypeLibrary if we need to
            cDLLResult = RegisterComponent(Filename, Unregister)
            If cDLLResult <> "" Then cTLBResult = RegisterTypeLibrary(Filename, Unregister)
            
            ' if both results are non-empty (failed), return the dll result, otherwise
            ' return "" (one of 'em worked)
            '
            If (cDLLResult <> "") And (cTLBResult <> "") Then
                RegisterFile = cDLLResult
            Else
                RegisterFile = cTLBResult
            End If
    End Select

End Function

Private Function RegisterTypeLibrary(ByVal Filename As String, ByVal Unregister As Boolean) As String

    Dim oTLIApp As TLIApplication
    
    On Error GoTo errHand
    
    Set oTLIApp = New TLIApplication
    If Unregister Then
        oTLIApp.TypeLibInfoFromFile(Filename).Unregister
    Else
        oTLIApp.TypeLibInfoFromFile(Filename).Register
    End If
    
    RegisterTypeLibrary = ""
    Exit Function
    
errHand:
    RegisterTypeLibrary = Err.Description
    Exit Function
End Function

Private Function RegisterComponent(ByVal Filename As String, ByVal Unregister As Boolean) As String
' calls DllRegisterServer/DllUnregisterServer

    Dim hMod As Long            ' module handle from LoadLibrary
    Dim cFunctionName As String ' function name
    Dim lpfn As Long            ' function pointer from GetProcAddress
    Dim hThread As Long         ' thread handle from CreateThread
    Dim nDummy As Long          ' dummy var for CreateThread
    Dim nResult As Long         ' WaitForSingleObject result
    Dim dwExitCode As Long      ' thread exit code (needed if we wait for too long) for ExitThread
    
    Dim cResult As String
    
    
    On Error GoTo errHand
    
    
    ' load the object into memory
    '
    hMod = LoadLibrary(Filename)
    If hMod <> 0 Then
    
        ' get the function name and address
        '
        If Unregister Then
            cFunctionName = "DllUnregisterServer"
        Else
            cFunctionName = "DllRegisterServer"
        End If
        lpfn = GetProcAddress(hMod, cFunctionName)
        If lpfn <> 0 Then
        
            ' call the function, and keep the thread id
            '
            hThread = CreateThread(ByVal 0&, ByVal 0&, ByVal lpfn, ByVal 0&, ByVal 0&, nDummy)
            If hThread <> 0 Then
                nResult = WaitForSingleObject(hThread, 10000)
                If nResult = WAIT_FAILED Then
                ' something odd happened
                    cResult = "WaitForSingleObject failed when calling " & cFunctionName & "."
                    Call GetExitCodeThread(hThread, dwExitCode)
                    Call ExitThread(dwExitCode)
                Else
                ' everything went just fine
                    cResult = ""
                End If
            Else ' hThread = 0
                cResult = cFunctionName & " was found, but the function could not be executed (CreateThread failed)."
            End If
        
            ' release the thread handle
            Call CloseHandle(hThread)
        
        Else ' lpfn = 0
            cResult = "File was loaded, but the " & cFunctionName & " entry point was not found."
        End If
        
        ' release the library
        FreeLibrary hMod
    
    Else ' hMod = 0
        cResult = "Not an exectuable file, or the file is corrupt (LoadLibrary failed)."
    End If
    
    
    ' return the result and leave
    '
    RegisterComponent = cResult
    Exit Function
    
errHand:
    RegisterComponent = "Error " & Err.Number & ": " & Err.Description
    Exit Function
End Function



'
' command-line, file functions
'
Private Function NormalizedCommandArray() As String()
' returns an array of strings that contains the command-line parameters
'
' assumes that: command-line parameters are separated by a space
'               if a parameter has an embedded space, the parameter is surrounded by quotes
' note that this format is how windows sends items on the command-line


    Dim cCommand As String
    Dim cChar As String
    Dim bInQuotes As Boolean
    Dim i As Long
    
    Dim aCommands() As String
    
    ' copy command line, replacing non-quoted spaces with a comma (which we should never get)
    '
    bInQuotes = False
    cCommand = ""
    
    For i = 1 To Len(Command$)
        cChar = Mid$(Command$, i, 1)
        Select Case Asc(cChar)
            
            Case 34 ' double quote
            
                ' toggle quote flag -- don't add
                bInQuotes = Not bInQuotes
                
            
            Case 32 ' space
                
                If bInQuotes Then
                ' pass unreplaced
                    cCommand = cCommand & cChar
                Else
                ' replace with comma
                    cCommand = cCommand & ","
                End If
                
            Case Else
            ' just add it
                cCommand = cCommand & cChar
        
        End Select
    Next ' i
    
    ' split on the commas
    aCommands = Split(cCommand, ",")
    
    ' do a little trim and send out
    For i = 0 To UBound(aCommands)
        aCommands(i) = Trim$(aCommands(i))
    Next ' i
    NormalizedCommandArray = aCommands

End Function

Private Function GetFileName(ByVal PathAndFile As String) As String
' returns the filename only.

    Dim cName As String
    Dim i As Long
    
    On Error GoTo errHand
    
    For i = Len(PathAndFile) To 1 Step -1
        Select Case Mid$(PathAndFile, i, 1)
            Case "\"
                Exit For
            Case Else
                cName = Mid$(PathAndFile, i, 1) & cName
        End Select
    Next ' i
    
    GetFileName = cName
    Exit Function
    
errHand:
    GetFileName = PathAndFile
    Exit Function
End Function


'
' shell functions
'
Private Sub SetAppPathKey(ByVal AppEXEName As String, ByVal AppPath As String)
' create the apppaths key (so we can be run from the command line)
    
    Dim cName As String
    Dim hKey As Long
    
    On Error Resume Next ' in case of reg permission errors
    
    ' fixup filename
    '
    If Right$(AppPath, 1) <> "\" Then AppPath = AppPath & "\"
    cName = AppPath & AppEXEName
    If LCase$(Right$(cName, 4)) <> ".exe" Then
        cName = cName & ".exe"
    Else
        cName = cName & ""
    End If
    
    ' denormalize path
    '
    AppPath = Left$(AppPath, Len(AppPath) - 1)
    
    ' add the registry entry
    '
    Call RegCreateKey(HKEY_LOCAL_MACHINE, "Software\Microsoft\Windows\CurrentVersion\App Paths\" & Dir$(cName), hKey)
    Call RegSetValueExStr(hKey, "", ByVal 0&, REG_SZ, cName, Len(cName))
    Call RegSetValueExStr(hKey, "Path", ByVal 0&, REG_SZ, AppPath, Len(AppPath))
    Call RegCloseKey(hKey)

    On Error GoTo 0

End Sub

