Attribute VB_Name = "modSupport"
Option Explicit

'
' modSupport.BAS
'
' support functions for the tab order addin
'
'

' API declarations
Private Const WM_SYSKEYDOWN = &H104
Private Const DI_NORMAL = 3

Public Declare Function SetWindowText Lib "user32" Alias "SetWindowTextA" (ByVal hWnd As Long, ByVal lpString As String) As Long

Private Declare Function PostMessage Lib "user32" Alias "PostMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function SetFocus Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetParent Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long

Private Declare Function DrawIconEx Lib "user32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal hIcon As Long, ByVal cxWidth As Long, ByVal cyWidth As Long, ByVal istepIfAniCur As Long, ByVal hbrFlickerFreeDraw As Long, ByVal diFlags As Long) As Long
Private Declare Function CreateIC Lib "gdi32" Alias "CreateICA" (ByVal lpDriverName As String, ByVal lpDeviceName As String, ByVal lpOutput As String, ByVal lpInitData As String) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hdc As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long


' global constants, vars
Public Const APP_CATEGORY = "Microsoft Visual Basic AddIns"

Public Const APP_NAME = "mike's vb6 tab order add-in"
Public Const PROG_ID = "mmTabOrder.TabOrderDocument"
Public Const MY_FAKE_GUID = "mike_mustaine_tab_order"
Public Const WINDOW_CAPTION = "Tab Order"


'
' ide functions
'
Public Function IDEInRunMode(ByVal IDEInstance As VBIDE.VBE) As Boolean

    On Error GoTo errHand

    IDEInRunMode = (IDEInstance.CommandBars("File").Controls(1).Enabled = False)
    Exit Function
    
errHand:
    IDEInRunMode = False
    Exit Function
End Function

Public Function GetAddin(ByVal IDEInstance As VBIDE.VBE, ByVal ProgID As String) As VBIDE.AddIn
' returns an addin with the specified progid, or Nothing

    On Error GoTo errHand
    
    Set GetAddin = IDEInstance.Addins(ProgID)
    Exit Function
    
errHand:
    Set GetAddin = Nothing
    Exit Function
End Function

Public Sub HandleKeyDown(ByVal IDEInstance As VBIDE.VBE, ByVal UserDoc As Object, ByRef KeyCode As Integer, ByVal Shift As Integer)
' pass system keystrokes back to the mdi parent (i think)
    
    Static hWndTop As Long
    
    ' leave if we shouldn't do anything
    If Shift <> vbAltMask Then Exit Sub
    If KeyCode < vbKeyA Then Exit Sub
    If KeyCode > vbKeyZ Then Exit Sub
    If IDEInstance Is Nothing Then Exit Sub
    If IDEInstance.DisplayModel = vbext_dm_SDI Then Exit Sub
    
    ' get the top level window, and post the key message to it
    If hWndTop = 0 Then hWndTop = FindHWndTop(UserDoc.hWnd)
    
    PostMessage hWndTop, WM_SYSKEYDOWN, KeyCode, &H20000000
    KeyCode = 0
    SetFocus hWndTop
    

End Sub

Private Function FindHWndTop(ByVal hWndChild As Long) As Long
' finds the top-level window for the passed in child

    Dim hWndParent As Long

    Do
        hWndParent = GetParent(hWndChild)
        If hWndParent = 0 Then
            FindHWndTop = hWndChild
            Exit Function
        End If
        hWndChild = hWndParent
    Loop

End Function

Public Function FindFloatingPalette(ByVal hWndChild As Long) As Long
' looks up the chain for a window with a class type of "VBFloatingPalette"
' and returns it or nothing


    Dim hWndParent As Long
    
    hWndParent = hWndChild
    Do
        If hWndParent = 0 Then
            FindFloatingPalette = 0
            Exit Function
        Else
            If LCase$(VBGetClassName(hWndParent)) = "vbfloatingpalette" Then
                FindFloatingPalette = hWndParent
                Exit Function
            End If
        End If
        hWndParent = GetParent(hWndParent)
    Loop


End Function

Private Function VBGetClassName(ByVal hWnd As Long) As String

    Dim cTemp As String
    Dim nLen As Long

    cTemp = String$(255, 0)
    nLen = GetClassName(hWnd, cTemp, Len(cTemp))
    VBGetClassName = Left$(cTemp, nLen)

End Function


'
' control display functions
'
Public Sub GetNameAndIndex(ByVal ListItem As String, ByRef Name As String, ByRef ArrayIndex As Long)
' returns the name and index from the passed listitem -- assumes that the listitem
' was generated using GetControlCaption()

    Dim nPos As Long
    Dim nEndPos As Long
    Dim cTemp As String
    

    ' remove any caption
    '
    nPos = InStr(1, ListItem, " ")
    If nPos > 0 Then
        cTemp = Left$(ListItem, nPos - 1)
    Else
        cTemp = ListItem
    End If
    
    ' see if there's an index, and handle it
    '
    nPos = InStr(1, cTemp, "(")
    If nPos > 0 Then
    ' index exists -- extract it, then clean up the string
        nEndPos = InStr(1, cTemp, ")")
        ArrayIndex = Val(Mid$(cTemp, nPos + 1, nEndPos - nPos))
        Name = Left$(cTemp, nPos - 1)
    Else
        ArrayIndex = -1
        Name = cTemp
    End If

End Sub

Public Function GetControlListItem(ByVal ctl As VBIDE.VBControl) As String
' returns a nice caption for the control

    Dim cName As String
    Dim cCaption As String
    Dim nIndex As Long
    
    Dim cListItem As String
    
    On Error Resume Next ' properties may be missing
    
    cName = ctl.Properties("Name")
    cCaption = ctl.Properties("Caption") ' will be blank if there's no caption
    nIndex = ctl.Properties("Index")
    
    cListItem = cName
    If nIndex > -1 Then cListItem = cListItem & "(" & nIndex & ")"
    If cCaption <> "" Then cListItem = cListItem & " - '" & cCaption & "'"
    
    GetControlListItem = cListItem
    On Error GoTo 0 ' restore default error handler

End Function


'
' control property functions
'
Public Function ControlFromListItem(ByVal ListItem As String, ByVal Controls As VBControls) As VBControl
' returns the control that matches the listiem text, or Nothing

    Dim cName As String
    Dim nIndex As Long

    On Error GoTo errHand
    
    GetNameAndIndex ListItem, cName, nIndex
    If nIndex > -1 Then
        Set ControlFromListItem = Controls(cName, nIndex)
    Else
        Set ControlFromListItem = Controls(cName)
    End If
    Exit Function
    
errHand:
    Set ControlFromListItem = Nothing
    Exit Function
End Function

Public Function ControlTabIndex(ByVal ctl As VBIDE.VBControl) As Long
' returns the tab index of the control, or -1

    On Error GoTo errHand
    
    ControlTabIndex = ctl.Properties("TabIndex")
    Exit Function
    
errHand:
    ControlTabIndex = -1
    Exit Function
End Function

Public Function ControlTop(ByVal ctl As VBIDE.VBControl) As Single
' returns the .Top property of the control, or -1

    On Error GoTo errHand
    
    ControlTop = ctl.Properties("Top")
    Exit Function
    
errHand:
    ControlTop = -1
    Exit Function
End Function

Public Function ControlLeft(ByVal ctl As VBIDE.VBControl) As Single
' returns the .Left property of the control, or -1

    On Error GoTo errHand
    
    ControlLeft = ctl.Properties("Left")
    Exit Function
    
errHand:
    ControlLeft = -1
    Exit Function
End Function


'
' listbox functions
'
Public Sub MoveListItem(ByVal lst As ListBox, ByVal SourceIndex As Long, ByVal DestIndex As Long)
' moves the listitem from sourceindex to destindex

    Dim cTemp As String
    
    With lst
    
        ' handle range errors
        If SourceIndex < 0 Then Exit Sub
        If DestIndex < 0 Then Exit Sub
        If SourceIndex > .ListCount - 1 Then Exit Sub
        If DestIndex > .ListCount - 1 Then Exit Sub
        If SourceIndex = DestIndex Then Exit Sub
        
        ' add a new copy of the source item,
        ' then remove the old one
        '
        If SourceIndex < DestIndex Then
            .AddItem .List(SourceIndex), DestIndex + 1
            .RemoveItem SourceIndex
        Else
            .AddItem .List(SourceIndex), DestIndex
            .RemoveItem SourceIndex + 1
        End If
    
    End With
    
End Sub

Public Sub SelectListIndex(ByVal lst As ListBox)
' ensure that the only listitem selected is the one
' that is the .ListIndex

    Dim i As Long
    Dim nListIndex As Long
    
    With lst
        nListIndex = .ListIndex
        For i = 0 To .ListCount - 1
            .Selected(i) = CBool(i = nListIndex)
        Next ' i
    End With

End Sub

Public Function IndexOfListItem(ByVal lst As ListBox, ByVal ListItem As String) As Long
' returns the index of the item, or -1

    Dim i As Long
    
    IndexOfListItem = -1 ' set default
    
    With lst
        For i = 0 To .ListCount - 1
            If .List(i) = ListItem Then
                IndexOfListItem = i
                Exit Function
            End If
        Next ' i
    End With
    

End Function

Public Function IndexOfListItemStartsWith(ByVal lst As ListBox, ByVal StartsWith As String) As Long
' a fuzzy search, just looking at the front of the list items

    Dim i As Long
    Dim nLength As Long
    
    IndexOfListItemStartsWith = -1
    nLength = Len(StartsWith)
    
    With lst
        For i = 0 To .ListCount - 1
            If Left$(.List(i), nLength) = StartsWith Then
                IndexOfListItemStartsWith = i
                Exit Function
            End If
        Next ' i
    End With
    
End Function


'
' generic functions
'
Public Function SafeBoolean(ByVal Value As String, ByVal DefaultValue As Boolean) As Boolean

    On Error GoTo errHand
    
    SafeBoolean = CBool(Value)
    Exit Function

errHand:
    SafeBoolean = DefaultValue
    Exit Function
End Function


'
' graphics functions
'
Public Sub PaintIconOnBitmap(ByVal SourceIcon As StdPicture, ByRef BackgroundBitmap As StdPicture)
' returns a stdpicure with the icon drawn on the background
'

    Dim hdc As Long
    Dim hBMPOld As Long
    
    hdc = CreateCompatibleDC(ByVal 0&)
    hBMPOld = SelectObject(hdc, BackgroundBitmap.Handle)
    Call DrawIconEx(hdc, 0, 0, SourceIcon.Handle, 16, 16, 0, 0, DI_NORMAL)
    
    Call SelectObject(hdc, hBMPOld)
    Call DeleteDC(hdc)
    
End Sub


